{
 Automatically locate the serial port that the Amicus18 is attached to
 Allow opening and closing of the Amicus18 virtual serial port
 Auto detect if the Amicus18 board is plugged into the USB socket
}
unit Amicus18_Coms;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Memo1: TMemo;
    OpenComsButton: TButton;
    CloseButton: TButton;
    CloseComsButton: TButton;
    Amicus18CheckTimer: TTimer;
    procedure CloseButtonClick(Sender: TObject);
    procedure OpenComsButtonClick(Sender: TObject);
    procedure CloseComsButtonClick(Sender: TObject);
    procedure Amicus18CheckTimerTimer(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  tClosePressed: Boolean;       // True if the close button is pressed

implementation

uses
    AmicusComPort;                  // Load the Amicus18 com port routines into the program

var
    tAmicus18Alive: Boolean;        // True if the Amicus18 board is still attached to the PC
    tNotFoundDisplayFlag: Boolean;
    tFoundDisplayFlag: Boolean;
{$R *.dfm}
//--------------------------------------------------------------------------------
// Form activate event
procedure TForm1.FormActivate(Sender: TObject);
begin
    OnActivate := Nil;
    tAmicus18Alive := False;
    
    tNotFoundDisplayFlag := False;
    tFoundDisplayFlag := False;
    Amicus18CheckTimer.Enabled := True;             // Start the timer event
end;
//--------------------------------------------------------------------------------
// Close button click event
procedure TForm1.CloseButtonClick(Sender: TObject);
begin
    tClosePressed := True;
    Close();
end;
//--------------------------------------------------------------------------------
// Open coms button click event
procedure TForm1.OpenComsButtonClick(Sender: TObject);
begin
    if(LocateAmicusComPort() = False) Then                  // Look for the Amicus18
        Memo1.Lines.Add('Cannot Find the Amicus18 board')
    else
        begin
            Memo1.Lines.Add('Amicus18 located on: ' + AmicusPort);
            If(OpenAmicusComPort(AmicusPort, 115200) = False) Then
                begin
                    OpenComsButton.Enabled := True;
                    CloseComsButton.Enabled := False;
                    Memo1.Lines.Add('Cannot Open the Amicus18 com port');
                end
            Else
                begin
                    OpenComsButton.Enabled := False;
                    CloseComsButton.Enabled := True;
                    Memo1.Lines.Add('Amicus18 Attached to: ' + AmicusPort + ' at 115200 baud');
                end;
        end;
end;
//--------------------------------------------------------------------------------
// Close coms button click event
procedure TForm1.CloseComsButtonClick(Sender: TObject);
begin
    //If(tAmicusPortOpen = False) Then Exit;
    
    CloseAmicusComPort();
    If(tAmicusPortOpen = False) Then
        begin
            OpenComsButton.Enabled := True;
            CloseComsButton.Enabled := False;
            Memo1.Lines.Add('Amicus18 Com port closed');
        end
    Else
        begin
            OpenComsButton.Enabled := False;
            CloseComsButton.Enabled := True;
            Memo1.Lines.Add('Cannot Close the Amicus18 Com port');
        end;
end;
//--------------------------------------------------------------------------------
// Timer event to look for the attachement and detachment of the Amicus18 board from USB
// If the Amicus18 board is not found, global variable tAmicus18Alive is false
//
procedure TForm1.Amicus18CheckTimerTimer(Sender: TObject);
begin
    tAmicus18Alive := LocateAmicusComPort();
    If(tAmicus18Alive = False) Then
        begin
            If(tNotFoundDisplayFlag = False) Then
                begin
                    tNotFoundDisplayFlag := True;
                    tFoundDisplayFlag := False;
                    OpenComsButton.Enabled := False;                // Disable the open coms button
                    CloseComsButton.Enabled := False;               // Disable the close coms button
                    Memo1.Lines.Add('Amicus18 board not found');    // Display a message
                end;
        end
    Else
        begin
            If(tFoundDisplayFlag = False) Then
                begin
                    tFoundDisplayFlag := True;
                    tNotFoundDisplayFlag := False;
                    OpenComsButton.Enabled := True;                 // Enable the open coms button
                    Memo1.Lines.Add('Amicus18 board found');        // Display a message
                end;
        end;
end;

end.
